'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.PivotTable = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _isString = require('d2-utilizr/lib/isString');

var _isString2 = _interopRequireDefault(_isString);

var _isNumber = require('d2-utilizr/lib/isNumber');

var _isNumber2 = _interopRequireDefault(_isNumber);

var _isNumeric = require('d2-utilizr/lib/isNumeric');

var _isNumeric2 = _interopRequireDefault(_isNumeric);

var _isObject = require('d2-utilizr/lib/isObject');

var _isObject2 = _interopRequireDefault(_isObject);

var _isBoolean = require('d2-utilizr/lib/isBoolean');

var _isBoolean2 = _interopRequireDefault(_isBoolean);

var _isEmpty = require('d2-utilizr/lib/isEmpty');

var _isEmpty2 = _interopRequireDefault(_isEmpty);

var _isArray = require('d2-utilizr/lib/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _numberConstrain = require('d2-utilizr/lib/numberConstrain');

var _numberConstrain2 = _interopRequireDefault(_numberConstrain);

var _objectApplyIf = require('d2-utilizr/lib/objectApplyIf');

var _objectApplyIf2 = _interopRequireDefault(_objectApplyIf);

var _arrayContains = require('d2-utilizr/lib/arrayContains');

var _arrayContains2 = _interopRequireDefault(_arrayContains);

var _arrayClean = require('d2-utilizr/lib/arrayClean');

var _arrayClean2 = _interopRequireDefault(_arrayClean);

var _ResponseRowIdCombination = require('../api/ResponseRowIdCombination');

var _colorUtils = require('../util/colorUtils');

var _PivotTableAxis = require('./PivotTableAxis');

var _PivotTableUtils = require('./PivotTableUtils');

var _PivotTableCells = require('./PivotTableCells');

var _PivotTableConstants = require('../table/PivotTableConstants');

var _PivotTableConstants2 = require('./PivotTableConstants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 *
 *
 * @param {object} refs
 * @param {object} layout
 * @param {object} response
 * @param {object} colAxis
 * @param {object} rowAxis
 * @param {object} [options={}]
 */
var PivotTable = exports.PivotTable = function PivotTable(refs, layout, response) {
    var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

    this.options = _extends({
        renderLimit: 100000,
        renderOffset: 1,
        cellHeight: 25,
        cellWidth: 120,
        trueTotals: false,
        unclickable: false,
        hideEmptyRows: !!layout.hideEmptyRows,
        hideEmptyColumns: !!layout.hideEmptyColumns,
        showDimensionLabels: !!layout.showDimensionLabels,
        showHierarchy: !!layout.showHierarchy,
        totalsAggregationType: layout.totalsAggregationType,
        numberType: layout.numberType,
        debug: {
            clipping: false
        }
    }, options);

    var colAxisOptions = {
        showSubTotals: !!layout.showRowSubTotals,
        showTotals: !!layout.showRowTotals,
        hideEmpty: !!layout.hideEmptyColumns
    };

    var rowAxisOptions = {
        showSubTotals: !!layout.showColSubTotals,
        showTotals: !!layout.showColTotals,
        hideEmpty: !!layout.hideEmptyRows
    };

    this.colAxis = new _PivotTableAxis.PivotTableAxis(refs, layout, response, _PivotTableConstants2.COLUMN_AXIS, colAxisOptions);
    this.rowAxis = new _PivotTableAxis.PivotTableAxis(refs, layout, response, _PivotTableConstants2.ROW_AXIS, rowAxisOptions);

    this.title = layout.title;
    this.digitGroupSeparator = layout.digitGroupSeparator;

    this.fontSize = layout.fontSize;
    this.displayDensity = layout.displayDensity;

    this.uuidDimUuidsMap = {};

    this.valueUuids = [];
    this.dimensionUuids = [];
    this.sortableIdObjects = [];

    this.filters = layout.filters;
    this.layout = layout;
    this.response = response;
    this.appManager = refs.appManager;
    this.optionConfig = refs.optionConfig;
    this.uiManager = refs.uiManager;

    this.legendSet = (0, _isObject2.default)(layout.legendSet) ? this.appManager.getLegendSetById(layout.legendSet.id) : null;

    this.legendDisplayStyle = layout.legendDisplayStyle;
    this.legendDisplayStrategy = layout.legendDisplayStrategy;

    this.valueCounter = 0;
};

PivotTable.prototype.initialize = function () {

    this.idValueMap = this.response.getIdMap(this.layout, 'value');

    // TODO: totalsAggregationType
    if (this.options.trueTotals) {
        this.idFactorMap = this.response.getIdMap(this.layout, 'factor');
        this.idNumeratorMap = this.response.getIdMap(this.layout, 'numerator');
        this.idDenominatorMap = this.response.getIdMap(this.layout, 'denominator');
        this.idMultiplierMap = this.response.getIdMap(this.layout, 'multiplier');
        this.idDivisorMap = this.response.getIdMap(this.layout, 'divisor');
    } else {
        this.idFactorMap = {};
        this.idNumeratorMap = {};
        this.idDenominatorMap = {};
        this.idMultiplierMap = {};
        this.idDivisorMap = {};
    }

    // lookup for values
    this.valueLookup = {};

    this.rowAxis.doTotals = this.rowAxis.doTotals && !!this.colAxis.size;
    this.colAxis.doTotals = this.colAxis.doTotals && !!this.rowAxis.size;

    this.colAxis.actualSize = this.colAxis.getActualSize();
    this.rowAxis.actualSize = this.rowAxis.getActualSize();

    // initialize lookup tables
    this.initializeLookups();

    // lookup for rows/columns to render (used to determine what rows/columns are hidden).
    if (this.doHideEmptyRows()) {
        this.rowAxis.initializeRenderLookup();
    }

    if (this.doHideEmptyColumns()) {
        this.colAxis.initializeRenderLookup();
    }
};

// setters

/**
 * Sets the index of where the table will start rendering columns.
 *
 * @param {number} columnIndex The index of the column to render
 */
PivotTable.prototype.setColumnStart = function (columnIndex) {
    this.columnStart = Math.max(0, columnIndex);
};

/**
 * Sets the index of where the table will start rendering rows.
 *
 * @param {number} rowIndex The index of the row to render
 */
PivotTable.prototype.setRowStart = function (rowIndex) {
    this.rowStart = Math.max(0, rowIndex);
};

/**
 * Sets the index of where the table will end rendering columns.
 *
 * @param {number} columnIndex The index of the column to render
 */
PivotTable.prototype.setColumnEnd = function (columnIndex) {
    this.columnEnd = this.constrainWidth(columnIndex);
};

/**
 * Sets the index of where the table will end rendering rows.
 *
 * @param {number} rowIndex The index of the row to render
 */
PivotTable.prototype.setRowEnd = function (rowIndex) {
    this.rowEnd = this.constrainHeight(rowIndex);
};

/**
 * Sets the size of the viewport and rebuilds the table.
 *
 * @param {number} [widthInPixels=0]
 * @param {number} [heightInPixels=0]
 */
PivotTable.prototype.setViewportSize = function (widthInPixels, heightInPixels) {
    this.viewportWidth = widthInPixels;
    this.viewportHeight = heightInPixels;
};

// options

PivotTable.prototype.doAverageTotals = function () {
    return this.options.totalsAggregationType === this.optionConfig.getTotalsAggregationType().average.id;
};

/**
 * Checks if column percentage option is enabled.
 *
 * @returns {boolean}
 */
PivotTable.prototype.doColumnPercentage = function () {
    return this.options.numberType === this.optionConfig.getNumberType().percentofcolumn.id;
};

/**
 * Checks if row percentage option is enabled.
 *
 * @returns {boolean}
 */
PivotTable.prototype.doRowPercentage = function () {
    return this.options.numberType === this.optionConfig.getNumberType().percentofrow.id;
};

/**
 * Checks if sortable column headers option is enabled.
 *
 * @returns
 */
PivotTable.prototype.doSortableColumnHeaders = function () {
    return this.rowAxis.type && this.rowAxis.dims === 1;
};

/**
 * Checks if hide empty columns option is enabled.
 *
 * @returns {boolean}
 */
PivotTable.prototype.doHideEmptyRows = function () {
    return this.options.hideEmptyRows && this.colAxis.type && this.rowAxis.type;
};

/**
 * Checks if hide empty rows option is enabled.
 *
 * @returns {boolean}
 */
PivotTable.prototype.doHideEmptyColumns = function () {
    return this.options.hideEmptyColumns && this.colAxis.type && this.rowAxis.type;
};

/**
 * Checks if show dimension labels option is enabled.
 *
 * @returns {boolean}
 */
PivotTable.prototype.doShowDimensionLabels = function () {
    return this.options.showDimensionLabels;
};

/**
 * Checks if dynamic table rendering option should be enabled.
 *
 * @returns {boolean}
 */
PivotTable.prototype.doClipping = function () {
    return this.response.getSize(this.layout) > this.options.renderLimit || this.options.debug.clipping;
};

/**
 * Checks if show hierarchy option should be enabled.
 *
 * @returns {boolean}
 */
PivotTable.prototype.doShowHierarchy = function () {
    return this.options.showHierarchy;
};

/**
 * Checks if legend display by data item option is enabled
 *
 * @returns {boolean}
 */
PivotTable.prototype.doLegendDisplayByDataItem = function () {
    return this.legendDisplayStrategy === this.getLegendDisplayStrategyId(_PivotTableConstants.BY_DATA_ITEM_STRATEGY);
};

/**
 * Checks if legend display option is enbalied
 *
 * @returns {boolean}
 */
PivotTable.prototype.doLegendDisplay = function () {
    return this.legendDisplayStrategy !== this.getLegendDisplayStrategyId(_PivotTableConstants.FIXED_STRATEGY);
};

/**
 * Checks if legend display style fill option is enabled
 *
 * @returns {boolean}
 */
PivotTable.prototype.doLegendDisplayStyleFill = function () {
    return this.legendDisplayStyle === this.getLegendDisplayStyleId(_PivotTableConstants.FILL_STYLE);
};

/**
 * Checks if legend display style text option is enabled
 *
 * @returns {boolean}
 */
PivotTable.prototype.doLegendDisplayStyleText = function () {
    return this.legendDisplayStyle === this.getLegendDisplayStyleId(_PivotTableConstants.TEXT_STYLE);
};

// checkers

/**
 * Checks if given cell is of type total
 *
 * @param {*} cell
 * @returns
 */
PivotTable.prototype.isDimensionTotalCell = function (cell) {
    return cell.type === _PivotTableConstants.DIMENSION_SUB_TOTAL_CELL || cell.type === _PivotTableConstants.DIMENSION_TOTAL_CELL;
};

/**
 * Checks if cell is valid (is a valid object and not collapsed)
 *
 * @param {object} cell
 * @returns
 */
PivotTable.prototype.isCellValid = function (cell) {
    return cell && !cell.collapsed;
};

/**
 * Checks if row located at row index is part of column axis.
 *
 * @param {number} columnIndex
 * @returns {boolean}
 */
PivotTable.prototype.isColumnDimensionRow = function (rowIndex) {
    return rowIndex < this.colAxis.dims;
};

/**
 * Checks if column located at column index is part of row axis.
 *
 * @param {number} columnIndex
 * @returns {boolean}
 */
PivotTable.prototype.isRowDimensionColumn = function (columnIndex) {
    return columnIndex < this.rowAxis.dims;
};

/**
 * Checks if given cell is of type text field.
 *
 * @param {string} type
 * @returns
 */
PivotTable.prototype.isTextCell = function (cell) {
    return !(0, _arrayContains2.default)(['dimension', 'filter'], cell.type);
};

// getters

/**
 * Gets the row index of where the value table starts relative to the
 * current state of the table.
 * Mostly used to offset the row index to skip over column
 * axis dimensions.
 *
 * @returns {number}
 */
PivotTable.prototype.getNumberOfVisibleColumnDimensions = function () {
    return Math.max(0, this.colAxis.dims - this.rowStart);
};

/**
 * Gets the column index of where the value table starts relative to the
 * current state of the table.
 * Mostly used to offset the column index to skip over row
 * axis dimensions.
 * @returns {number}
 */
PivotTable.prototype.getNumberOfVisibleRowDimensions = function () {
    return Math.max(0, this.rowAxis.dims - this.columnStart);
};

/**
 * TODO: This needs a better name
 *
 * @param {number} numerator
 * @param {number} denominator
 * @param {number} multiplier
 * @param {number} divisor
 * @returns
 */
PivotTable.prototype.getAverageTotal = function (numerator, denominator, multiplier, divisor) {
    return numerator * multiplier / (denominator * divisor || 1);
};
// PivotTable.prototype.getAverageTotal = function(numerator, denominator, multiplier, divisor) {
//     return (numerator * multiplier) / ((denominator || 1) * divisor);
// };

/**
 * TODO: This needs a better name
 *
 * @param {number} numerator
 * @param {number} denominator
 * @param {number} factor
 * @param {number} counter
 * @returns
 */
PivotTable.prototype.getSumTotal = function (numerator, denominator, factor, counter) {
    return numerator * (factor || 1) / ((denominator || 1) * counter);
};

/**
 * Gets the html value of a table cell.
 *
 * @param {object} cell
 * @returns {string}
 */
PivotTable.prototype.getDisplayValue = function (cell) {

    if (!this.isCellValid(cell)) {
        return '';
    }

    return this.isTextCell(cell) ? this.getPrettyHtml(cell.displayValue) : cell.displayValue;
};

/**
 * gets pretty print of given string.
 *
 * @param {string} displayValue
 * @returns {string}
 */
PivotTable.prototype.getPrettyHtml = function (displayValue) {
    return displayValue === null || displayValue === undefined ? '' : this.optionConfig.prettyPrint(displayValue, this.digitGroupSeparator);
};

/**
 * Gets legend display strategy id.
 *
 * @param {string} type
 * @returns {number}
 */
PivotTable.prototype.getLegendDisplayStrategyId = function (type) {
    return this.optionConfig.getLegendDisplayStrategy(type).id;
};

/**
 * Gets legend display style id.
 *
 * @param {string} type
 * @returns {number}
 */
PivotTable.prototype.getLegendDisplayStyleId = function (type) {
    return this.optionConfig.getLegendDisplayStyle(type).id;
};

/**
 * Gets the number of cells that can be rendered horizontally within the viewport.
 *
 * @returns {number}
 */
PivotTable.prototype.getViewportWidthIndex = function () {
    return Math.floor(this.viewportWidth / this.options.cellWidth) + this.options.renderOffset;
};

/**
 * Gets the number of cells that can be rendered vertically within the viewport.
 * @returns {number}
 */
PivotTable.prototype.getViewportHeightIndex = function () {
    return Math.floor(this.viewportHeight / this.options.cellHeight) + this.options.renderOffset;
};

/**
 * Gets the height of the entire table in number of cells,
 * including total and sub total rows.
 *
 * @returns {number}
 */
PivotTable.prototype.getRowAxisSize = function () {
    return this.rowAxis.actualSize + this.colAxis.dims - 1;
};

/**
 * Gets the width of the entire table in number of cells
 * including total and sub total columns.
 *
 * @returns {number}
 */
PivotTable.prototype.getColumnAxisSize = function () {
    return this.colAxis.actualSize + this.rowAxis.dims - 1;
};

/**
 * Gets the legend set id.
 *
 * @param {string} dxId
 * @returns {string}
 */
PivotTable.prototype.getLegendSetId = function (dxId) {
    return this.response.metaData.items[dxId].legendSet;
};

/**
 * Gets a value object used to calcualte correct totals
 * and fill up the value lookup table.
 *
 * @param {number} rowIndex row index of value
 * @param {number} columnIndex column index of value
 * @returns {object}
 */
PivotTable.prototype.getValueObject = function (rowIndex, columnIndex) {

    rowIndex = this.rowAxis.getPositionIndexWithoutTotals(rowIndex);
    columnIndex = this.colAxis.getPositionIndexWithoutTotals(columnIndex);

    var rric = new _ResponseRowIdCombination.ResponseRowIdCombination();

    if (this.colAxis.type) {
        var columnId = this.colAxis.ids[columnIndex];
        rric.add(columnId);
    }

    if (this.rowAxis.type) {
        var rowId = this.rowAxis.ids[rowIndex];
        rric.add(rowId);
    }

    var id = rric.get();
    var value = this.getValueFromId(id);
    var empty = value === null;

    var valueObject = {
        empty: empty,
        value: empty ? 0 : value,
        numerator: this.getNumeratorValue(id),
        denominator: this.getDenominatorValue(id),
        factor: this.getFactorValue(id),
        multiplier: this.getMultiplierValue(id),
        divisor: this.getDivisorValue(id)
    };

    if (this.response.getDxIds()) {
        var dxId = this.response.getDxIds().find(function (dxId) {
            return id.includes(dxId);
        });

        valueObject['totalAggregationType'] = this.response.getTotalAggregationType(dxId);

        valueObject['valueType'] = this.response.getValueType(dxId);
    }

    return valueObject;
};

/**
 * Gets value based on given ID.
 *
 * @param {string} id
 * @returns {number}
 */
PivotTable.prototype.getValueFromId = function (id) {

    var value = this.idValueMap[id];

    if ((0, _isBoolean2.default)(value)) {
        return 1;
    }

    if (value === null || typeof value === 'undefined') {
        return null;
    }

    return (0, _isNumeric2.default)(value) ? parseFloat(value) : value;
};
// PivotTable.prototype.getValueFromId = function(id) {

//     let value = this.idValueMap[id];

//     if (isBoolean(value)) {
//         return 1;
//     }

//     value = parseFloat(value);

//     if (!isNumber(value)) {
//         return  null;
//     }

//     return value;
// };
/**
 * Gets factor value from id factor map
 *
 * @param {string} id
 * @returns {number}
 */
PivotTable.prototype.getFactorValue = function (id) {
    return parseFloat(this.idFactorMap[id]) || 1;
};
/**
 * Gets numerator value from id numerator map
 *
 * @param {string} id
 * @returns {number}
 */
PivotTable.prototype.getNumeratorValue = function (id) {
    return parseFloat(this.idNumeratorMap[id]) || this.getValueFromId(id);
};
/**
 * Gets denominator value from id denominator map
 *
 * @param {string} id
 * @returns {number}
 */
PivotTable.prototype.getDenominatorValue = function (id) {
    return parseFloat(this.idDenominatorMap[id]) || 0;
};
/**
 * Gets multiplier value from id multiplier map
 *
 * @param {string} id
 * @returns {number}
 */
PivotTable.prototype.getMultiplierValue = function (id) {
    return parseFloat(this.idMultiplierMap[id]) || 0;
};
/**
 * Gets divisor value from id divisor map
 *
 * @param {string} id
 * @returns {number}
 */
PivotTable.prototype.getDivisorValue = function (id) {
    return parseFloat(this.idDivisorMap[id]) || 0;
};

/**
 * Gets uuids object map.
 *
 * @returns {object}
 */
PivotTable.prototype.getUuidObjectMap = function () {
    return (0, _objectApplyIf2.default)(this.colAxis ? this.colAxis.uuidObjectMap : {}, this.rowAxis ? this.rowAxis.uuidObjectMap : {});
};

/**
 * Gets the index of the last cell to be rendered in column.
 *
 * @param {number} columnStart
 * @returns {number}
 */
PivotTable.prototype.getColumnEnd = function (columnStart) {

    if (this.doClipping()) {
        return this.constrainWidth(this.getViewportWidthIndex() + columnStart);
    }

    return this.getColumnAxisSize() - this.colAxis.numberOfEmpty;
};

/**
 * Gets the index of the last cell to be rendered in row.
 *
 * @param {number} rowStart
 * @returns {number}
 */
PivotTable.prototype.getRowEnd = function (rowStart) {

    if (this.doClipping()) {
        return this.constrainHeight(this.getViewportHeightIndex() + rowStart);
    }

    return this.getRowAxisSize() - this.rowAxis.numberOfEmpty;
};

/**
 * Gets the row sort id
 *
 * @param {number} rowIndex
 * @returns {string}
 */
PivotTable.prototype.getRowSortId = function (rowIndex, columnIndex) {

    if (this.doSortableColumnHeaders() && rowIndex === this.colAxis.dims - 1) {
        return this.colAxis.ids[columnIndex];
    }

    return null;
};

/**
 * Gets the label for the column axis at given index.
 * TODO: Remove?
 * @param {number} rowIndex
 * @returns {string}
 */
PivotTable.prototype.getColumnAxisLabel = function (rowIndex) {

    if (this.colAxis.dims) {
        return this.response.getNameById(this.colAxis.dimensionNames[rowIndex]);
    }

    return null;
};

/**
 * Gets the dimension label for the row axis at given index
 * TODO: Remove?
 * @param {number} columnIndex
 * @returns {string}
 */
PivotTable.prototype.getRowAxisLabel = function (columnIndex) {

    if (this.rowAxis.dims) {
        return this.response.getNameById(this.rowAxis.dimensionNames[columnIndex]);
    }

    return null;
};

/**
 * Gets the dimension label for the shared column/row.
 *
 * @returns {string}
 */
PivotTable.prototype.getCrossAxisLabel = function () {

    var colAxisLabel = this.getColumnAxisLabel(this.colAxis.dims - 1);
    var rowAxisLabel = this.getRowAxisLabel(this.rowAxis.dims - 1);

    if (!this.rowAxis.type) {
        return colAxisLabel;
    }

    if (!this.colAxis.type) {
        return rowAxisLabel;
    }

    if (colAxisLabel) {
        rowAxisLabel += _PivotTableConstants.NO_BREAK_SPACE + '/' + _PivotTableConstants.NO_BREAK_SPACE + colAxisLabel;
    }

    return rowAxisLabel;
};

/**
 * Gets the span for the top title bar.
 *
 * @param {number} span
 * @returns {number}
 */
PivotTable.prototype.getTopBarSpan = function (span) {
    var rowDims = this.rowAxis.dims || 0;

    if (!this.colAxis.type && this.rowAxis.type) {
        return rowDims;
    }

    return span;
};

// Table Modifiers
/**
 * Adds a column to the right side of the table.
 *
 * @param {number} columnIndex index of column to render
 * @param {number} rowStart index of the first row of the column
 * @param {number} rowEnd  index of the last row of the column
 */
PivotTable.prototype.addRightColumn = function (columnIndex, rowStart, rowEnd) {

    var column = this.buildColumn(columnIndex, rowStart, rowEnd);

    for (var rowIndex = 0; rowIndex < column.length; rowIndex++) {
        this.table[rowIndex].push(column[rowIndex]);
    }
};

/**
 * Adds a column to the left side of the table.
 *
 * @param {number} columnIndex index of column to render
 * @param {number} rowStart index of the first row of the column
 * @param {number} rowEnd index of the last row of the column
 */
PivotTable.prototype.addLeftColumn = function (columnIndex, rowStart, rowEnd) {

    var column = this.buildColumn(columnIndex, rowStart, rowEnd);

    for (var rowIndex = 0; rowIndex < column.length; rowIndex++) {
        this.table[rowIndex].unshift(column[rowIndex]);
    }
};

/**
 * Adds a row to the top side of the table.
 *
 * @param {number} rowIndex index of row to render
 * @param {number} columnStart index of the first column of the row
 * @param {number} columnEnd index of the last column of the row
 */
PivotTable.prototype.addTopRow = function (rowIndex, columnStart, columnEnd) {
    this.table.unshift(this.buildRow(rowIndex, columnStart, columnEnd));
};

/**
 * Adds a row to the bottom side of the table.
 *
 * @param {number} rowIndex index of row to render
 * @param {number} columnStart index of the first column of the row
 * @param {number} columnEnd index of the last column of the row
 */
PivotTable.prototype.addBottomRow = function (rowIndex, columnStart, columnEnd) {
    this.table.push(this.buildRow(rowIndex, columnStart, columnEnd));
};

/**
 * Removes a column from the left side of the table.
 *
 */
PivotTable.prototype.deleteLeftColumn = function () {
    for (var rowIndex = 0; rowIndex < this.table.length; rowIndex++) {
        this.table[rowIndex].shift();
    }
};

/**
 * Removes a column from the right side of the table.
 *
 */
PivotTable.prototype.deleteRightColumn = function () {
    for (var rowIndex = 0; rowIndex < this.table.length; rowIndex++) {
        this.table[rowIndex].pop();
    }
};

/**
 * Removes a column from the top side of the table.
 *
 */
PivotTable.prototype.deleteTopRow = function () {
    this.table.shift();
};

/**
 * Removes a column from the bottom side of the table.
 *
 */
PivotTable.prototype.deleteBottomRow = function () {
    this.table.pop();
};

/**
 * Removes row of given index from the table.
 *
 * @param {number} rowIndex index of row to be removed
 */
PivotTable.prototype.deleteRow = function (rowIndex) {
    this.table.splice(rowIndex, 1);
};

/**
 * Removes column of given index from the table.
 *
 * @param {number} columnIndex index of column to be removed
 */
PivotTable.prototype.deleteColumn = function (columnIndex) {
    for (var rowIndex = 0; rowIndex < this.table.length; rowIndex++) {
        this.table[rowIndex].splice(columnIndex, 1);
    }
};

/**
 * Builds a table row based on given parameters.
 *
 * @param {number} rowIndex
 * @param {number} columnStart
 * @param {number} columnEnd
 * @returns {array}
 */
PivotTable.prototype.buildRow = function (rowIndex, columnStart, columnEnd) {

    var rowAxisRow = [];

    if (this.isColumnDimensionRow(rowIndex)) {
        return this.buildColumnDimensionRow(rowIndex, columnStart, columnEnd);
    }

    rowIndex -= this.colAxis.dims;

    if (this.isRowDimensionColumn(columnStart)) {
        rowAxisRow = this.buildRowDimensionRow(rowIndex, columnStart);
    }

    columnStart = Math.max(columnStart - this.rowAxis.dims, 0);
    columnEnd -= this.rowAxis.dims;

    return rowAxisRow.concat(this.buildValueRow(rowIndex, columnStart, columnEnd));
};

/**
 * Builds a table column based on given parameters
 *
 * @param {number} columnIndex
 * @param {number} rowStart
 * @param {number} rowEnd
 * @returns {array}
 */
PivotTable.prototype.buildColumn = function (columnIndex, rowStart, rowEnd) {

    // Build column axis (will create corner axis if column index is over row axis).
    var columnAxis = this.buildColumnDimensionColumn(columnIndex, rowStart);

    if (this.isRowDimensionColumn(columnIndex)) {
        var rowAxis = this.buildRowDimensionColumn(columnIndex, rowStart, rowEnd);
        return columnAxis.concat(rowAxis);
    }

    return columnAxis.concat(this.buildValueColumn(columnIndex, rowStart, rowEnd));
};

/**
 * Builds value portion of table row based on given parameters.
 *
 * @param {number} rowIndex
 * @param {number} columnStart
 * @param {number} columnEnd
 * @returns {array}
 */
PivotTable.prototype.buildValueRow = function (rowIndex, columnStart, columnEnd) {
    var row = [];

    for (var columnIndex = columnStart; columnIndex <= columnEnd; columnIndex++) {
        row.push(this.buildValueCell(columnIndex, rowIndex));
    }

    return row;
};

/**
 * Builds value portion of table column based on given parameters.
 *
 * @param {number} columnIndex
 * @param {number} rowStart
 * @param {number} rowEnd
 * @returns {array}
 */
PivotTable.prototype.buildValueColumn = function (columnIndex, rowStart, rowEnd) {
    var column = [];

    // offset row dimension
    columnIndex = Math.max(columnIndex - this.rowAxis.dims, 0);

    rowStart = Math.max(rowStart - this.colAxis.dims, 0);
    rowEnd -= this.colAxis.dims;

    for (var rowIndex = rowStart; rowIndex <= rowEnd; rowIndex++) {
        column.push(this.buildValueCell(columnIndex, rowIndex));
    }

    return column;
};

/**
 * Builds value cell based on row and column index pair.
 *
 * @param {number} columnIndex
 * @param {number} rowIndex
 * @returns {object}
 */
PivotTable.prototype.buildValueCell = function (columnIndex, rowIndex) {

    rowIndex = this.rowAxis.getPositionIndexOffsetHidden(rowIndex);
    columnIndex = this.colAxis.getPositionIndexOffsetHidden(columnIndex);

    var valueObject = this.valueLookup[rowIndex][columnIndex];

    var value = (0, _isObject2.default)(valueObject) ? valueObject.value : value;
    var displayValue = value;
    // let value = this.valueLookup[rowIndex][columnIndex].value;

    if (this.doColumnPercentage() && (0, _isNumber2.default)(value)) {
        displayValue = this.colAxis.getPercentage(value, columnIndex);
    }

    if (this.doRowPercentage() && (0, _isNumber2.default)(value)) {
        displayValue = this.rowAxis.getPercentage(value, rowIndex);
    }

    if (this.rowAxis.isTotalPosition(rowIndex) || this.colAxis.isTotalPosition(columnIndex)) {
        var totalObj = this.totalMap[rowIndex][columnIndex];

        displayValue = (0, _PivotTableUtils.getDefaultNumberDisplayValue)(displayValue, this.layout.skipRounding);
        displayValue = this.getPrettyHtml(displayValue);

        return new _PivotTableCells.TotalCell(value, displayValue, _extends({}, totalObj, { skipRounding: this.layout.skipRounding }));
    }

    if (this.colAxis.isSubTotalPosition(columnIndex) || this.rowAxis.isSubTotalPosition(rowIndex)) {
        var _totalObj = this.totalMap[rowIndex][columnIndex];

        displayValue = (0, _PivotTableUtils.getDefaultNumberDisplayValue)(displayValue, this.layout.skipRounding);
        displayValue = this.getPrettyHtml(displayValue);

        return new _PivotTableCells.SubTotalCell(value, displayValue, _extends({}, _totalObj, { skipRounding: this.layout.skipRounding }));
    }

    if (value === null || typeof value === 'undefined') {
        displayValue = this.getPrettyHtml(displayValue);

        return new _PivotTableCells.PlainValueCell(value, displayValue, { skipRounding: this.layout.skipRounding });
    }

    var cell = void 0;

    if (valueObject.valueType === _PivotTableConstants.NUMBER_VALUE_TYPE) {
        displayValue = (0, _PivotTableUtils.getDefaultNumberDisplayValue)(displayValue, this.layout.skipRounding);
        displayValue = this.getPrettyHtml(displayValue);

        cell = new _PivotTableCells.ValueCell(value, displayValue, { skipRounding: this.layout.skipRounding });
    } else {
        cell = new _PivotTableCells.TextValueCell(value, displayValue, { skipRounding: this.layout.skipRounding });
    }

    rowIndex = this.rowAxis.getPositionIndexWithoutTotals(rowIndex);
    columnIndex = this.colAxis.getPositionIndexWithoutTotals(columnIndex);

    var rric = new _ResponseRowIdCombination.ResponseRowIdCombination();

    this.colAxis.type && rric.add(this.colAxis.ids[columnIndex]);
    this.rowAxis.type && rric.add(this.rowAxis.ids[rowIndex]);

    cell.dxId = rric.getIdByIds(this.response.metaData.dimensions.dx);
    cell.peId = rric.getIdByIds(this.response.metaData.dimensions.pe);
    cell.ouId = rric.getIdByIds(this.response.metaData.dimensions.ou);

    cell.uuids = [];

    if (this.colAxis.type) {
        cell.uuids = cell.uuids.concat(this.colAxis.objects.all[this.colAxis.dims - 1][columnIndex].uuids);
    }

    if (this.rowAxis.type) {
        cell.uuids = cell.uuids.concat(this.rowAxis.objects.all[this.rowAxis.dims - 1][rowIndex].uuids);
    }

    this.uuidDimUuidsMap[cell.uuid] = cell.uuids;

    return cell;
};

/**
 * Builds row axis dimension based on given parameters.
 *
 * @returns {array}
 */
PivotTable.prototype.buildRowDimension = function () {

    var axis = [];

    if (!this.rowAxis.type) {

        if (this.doShowDimensionLabels()) {
            axis[0] = [new _PivotTableCells.DimensionEmptyCell({ style: 'visibility: hidden;' })];
        }

        return axis;
    }

    for (var rowIndex = this.rowStart; rowIndex <= this.rowEnd - this.colAxis.dims; rowIndex++) {
        axis.push(this.buildRowDimensionRow(rowIndex, this.columnStart));
    }

    return axis;
};

/**
 * Builds row axis dimension row based on given parameters.
 *
 * @param {number} rowIndex
 * @param {number} columnStart
 * @returns {array}
 */
PivotTable.prototype.buildRowDimensionRow = function (rowIndex, columnStart) {

    if (this.rowAxis.dims < columnStart) {
        return [];
    }

    if (!this.rowAxis.type) {
        return [new _PivotTableCells.DimensionEmptyCell({ style: 'visibility: hidden;' })];
    }

    var row = [];

    for (var columnIndex = columnStart; columnIndex < this.rowAxis.dims; columnIndex++) {
        row.push(this.buildRowDimensionCell(columnIndex, rowIndex));
    }

    return row;
};

/**
 * Builds column axis dimension row based on given parameters.
 *
 * @param {number} rowIndex
 * @param {number} columnStart
 * @param {number} columnEnd
 * @returns {array}
 */
PivotTable.prototype.buildColumnDimensionRow = function (rowIndex, columnStart, columnEnd) {

    var row = [];

    if (this.colAxis.dims < rowIndex) {
        return row;
    }

    if (this.isRowDimensionColumn(columnStart)) {
        _extends(row, this.buildCornerDimensionRow(rowIndex, columnStart));
    }

    if (!this.colAxis.type) {
        return row;
    }

    columnStart += row.length;

    columnStart -= this.rowAxis.dims;
    columnEnd -= this.rowAxis.dims;

    for (var columnIndex = row.length; columnStart <= columnEnd; columnIndex++, columnStart++) {
        row[columnIndex] = this.buildColumnDimensionCell(rowIndex, columnStart);
    }

    return row;
};

/**
 * Builds row axis dimension column based on given parameters.
 *
 * @param {number} columnIndex
 * @param {number} rowStart
 * @param {number} rowEnd
 * @returns {array}
 */
PivotTable.prototype.buildRowDimensionColumn = function (columnIndex, rowStart, rowEnd) {

    var column = [];

    // if requested column at index is not visible return empty array
    if (this.rowAxis.dims < columnIndex) {
        return column;
    }

    // if there is no axis dimension fill with blank invisible cells
    if (!this.rowAxis.type) {

        for (var rowIndex = 0; rowIndex <= rowEnd - rowStart; rowIndex++) {
            column[rowIndex] = new _PivotTableCells.DimensionEmptyCell({ style: 'visibility: hidden;' });
        }

        return column;
    }

    // offset column dimension
    rowStart = Math.max(rowStart - this.colAxis.dims, 0);
    rowEnd -= this.colAxis.dims;

    for (var _rowIndex = rowStart; _rowIndex <= rowEnd; _rowIndex++) {
        column.push(this.buildRowDimensionCell(columnIndex, _rowIndex));
    }

    return column;
};

/**
 * Builds row axis cell based on row and column index pair.
 *
 * @param {number} columnIndex
 * @param {number} rowIndex
 * @returns {object}
 */
PivotTable.prototype.buildRowDimensionCell = function (columnIndex, rowIndex) {

    rowIndex = this.rowAxis.getPositionIndexOffsetHidden(rowIndex);

    var config = {};
    var displayValue = _PivotTableConstants.NO_BREAK_SPACE;

    if (this.rowAxis.isSubTotalPosition(rowIndex)) {

        config.colSpan = this.rowAxis.dims - columnIndex;
        config.hidden = columnIndex !== this.columnStart;

        return new _PivotTableCells.DimensionSubTotalCell(displayValue, config);
    }

    if (this.rowAxis.isTotalPosition(rowIndex)) {

        config.colSpan = this.rowAxis.dims - columnIndex;

        if (columnIndex !== 0) {
            config.sort = _PivotTableConstants.TOTAL_SORT;
            config.generateUuid = true;
            config.hidden = true;
        }

        displayValue = columnIndex === 0 ? 'Total' : _PivotTableConstants.NO_BREAK_SPACE;

        return new _PivotTableCells.DimensionGrandTotalCell(displayValue, config);
    }

    rowIndex = this.rowAxis.getPositionIndexWithoutTotals(rowIndex);

    config = this.rowAxis.objects.all[columnIndex][rowIndex];
    config.showHierarchy = this.doShowHierarchy();

    displayValue = this.response.getItemName(config.id, config.showHierarchy, true);

    return new _PivotTableCells.RowAxisCell(displayValue, config);
};

/**
 * Builds column axis cell based on row and column index pair
 *
 * @param {number} rowIndex
 * @param {number} columnIndex
 * @returns {object}
 */
PivotTable.prototype.buildColumnDimensionCell = function (rowIndex, columnIndex) {

    columnIndex = this.colAxis.getPositionIndexOffsetHidden(columnIndex);

    var config = {};
    var displayValue = _PivotTableConstants.NO_BREAK_SPACE;

    if (this.colAxis.isSubTotalPosition(columnIndex)) {

        config.hidden = rowIndex !== this.rowStart;
        config.rowSpan = this.colAxis.dims - rowIndex;

        return new _PivotTableCells.DimensionSubTotalCell(displayValue, config);
    }

    if (this.colAxis.isTotalPosition(columnIndex)) {

        config.hidden = rowIndex !== this.rowStart;
        config.rowSpan = this.colAxis.dims - rowIndex;
        config.sort = rowIndex === this.rowStart && this.doSortableColumnHeaders() ? _PivotTableConstants.TOTAL_SORT : null;
        config.generateUuid = rowIndex === 0;

        displayValue = rowIndex === 0 ? 'Total' : _PivotTableConstants.NO_BREAK_SPACE;

        return new _PivotTableCells.DimensionGrandTotalCell(displayValue, config);
    }

    columnIndex = this.colAxis.getPositionIndexWithoutTotals(columnIndex);

    config = this.colAxis.objects.all[rowIndex][columnIndex];

    config.showHierarchy = this.doShowHierarchy();
    config.sort = this.getRowSortId(rowIndex, columnIndex);

    displayValue = this.response.getItemName(config.id, config.showHierarchy, true);

    return new _PivotTableCells.ColumnAxisCell(displayValue, config);
};

/**
 * Builds column axis dimension based on given parameters
 *
 * @param {number} columnStart
 * @param {number} columnEnd
 * @param {number} rowStart
 * @returns {array}
 */
PivotTable.prototype.buildColumnDimension = function () {

    if (!this.colAxis.type) {
        return this.buildCornerDimensionRow(0, 0);
    }

    var column = [];

    for (var columnIndex = this.columnStart; columnIndex <= this.columnEnd; columnIndex++) {
        column.push(this.buildColumnDimensionColumn(columnIndex, this.rowStart));
    }

    return column;
};

/**
 * Builds column axis dimension column based on given parameters
 *
 * @param {number} columnIndex
 * @param {number} rowStart
 * @returns {array}
 */
PivotTable.prototype.buildColumnDimensionColumn = function (columnIndex, rowStart) {

    if (this.colAxis.dims < rowStart) {
        return [];
    }

    if (this.isRowDimensionColumn(columnIndex)) {
        return this.buildCornerDimensionColumn(columnIndex, rowStart);
    }

    columnIndex -= this.rowAxis.dims;

    var column = [];

    for (var rowIndex = rowStart; rowIndex < this.colAxis.dims; rowIndex++) {
        column.push(this.buildColumnDimensionCell(rowIndex, columnIndex));
    }

    return column;
};

/**
 * Builds corner axis dimension.
 *
 * @returns {array}
 */
PivotTable.prototype.buildCornerDimension = function () {
    var cornerAxis = [];

    for (var rowIndex = 0; rowIndex < this.colAxis.dims; rowIndex++) {
        cornerAxis[rowIndex] = this.buildCornerDimensionRow(rowIndex, 0);
    }

    return cornerAxis;
};

/**
 * Builds corner axis dimension column based on given parameters.
 *
 * @param {number} columnIndex
 * @param {number} rowStart
 * @returns {array}
 */
PivotTable.prototype.buildCornerDimensionColumn = function (columnIndex, rowStart) {

    var column = [];

    if (!this.doShowDimensionLabels()) {

        var colSpan = this.rowAxis.dims - columnIndex;
        var rowSpan = this.colAxis.dims - rowStart;

        var hidden = columnIndex !== this.columnStart;

        column.push(new _PivotTableCells.DimensionEmptyCell({ colSpan: colSpan, rowSpan: rowSpan, hidden: hidden }));

        for (var rowIndex = rowStart + 1; rowIndex < this.colAxis.dims; rowIndex++) {
            rowSpan = this.colAxis.dims - rowIndex;
            column.push(new _PivotTableCells.DimensionEmptyCell({ colSpan: colSpan, rowSpan: rowSpan, hidden: true }));
        }

        return column;
    }

    if (columnIndex === this.rowAxis.dims - 1) {

        for (var _rowIndex2 = rowStart; _rowIndex2 < this.colAxis.dims - 1; _rowIndex2++) {
            column.push(new _PivotTableCells.DimensionLabelCell(this.getColumnAxisLabel(_rowIndex2)));
        }

        column.push(new _PivotTableCells.DimensionLabelCell(this.getCrossAxisLabel()));

        return column;
    }

    for (var _rowIndex3 = rowStart; _rowIndex3 < this.colAxis.dims - 1; _rowIndex3++) {
        column.push(new _PivotTableCells.DimensionLabelCell(_PivotTableConstants.NO_BREAK_SPACE));
    }

    column.push(new _PivotTableCells.DimensionLabelCell(this.getRowAxisLabel(columnIndex)));

    return column;
};

/**
 * Builds corner axis dimension row based on given parameters.
 *
 * @param {number} rowIndex
 * @param {number} columnStart
 * @returns {array}
 */
PivotTable.prototype.buildCornerDimensionRow = function (rowIndex, columnStart) {

    var row = [];

    if (!this.doShowDimensionLabels()) {

        var colSpan = this.rowAxis.dims - columnStart;
        var rowSpan = this.colAxis.dims - rowIndex;

        var hidden = columnStart === this.columnStart;

        row.push(new _PivotTableCells.DimensionEmptyCell({ colSpan: colSpan, rowSpan: rowSpan, hidden: hidden }));

        for (var columnIndex = columnStart + 1; columnIndex < this.rowAxis.dims; columnIndex++) {
            colSpan = this.rowAxis.dims - columnIndex;
            row.push(new _PivotTableCells.DimensionEmptyCell({ colSpan: colSpan, rowSpan: rowSpan, hidden: true }));
        }

        return row;
    }

    if (rowIndex === this.colAxis.dims - 1 || this.colAxis.dims === 0) {

        for (var _columnIndex = columnStart; _columnIndex < this.rowAxis.dims - 1; _columnIndex++) {
            row.push(new _PivotTableCells.DimensionLabelCell(this.getRowAxisLabel(_columnIndex)));
        }

        row.push(new _PivotTableCells.DimensionLabelCell(this.getCrossAxisLabel()));

        return row;
    }

    for (var _columnIndex2 = columnStart; _columnIndex2 < this.rowAxis.dims - 1; _columnIndex2++) {
        row.push(new _PivotTableCells.DimensionLabelCell(_PivotTableConstants.NO_BREAK_SPACE));
    }

    row.push(new _PivotTableCells.DimensionLabelCell(this.getColumnAxisLabel(rowIndex)));

    return row;
};

/**
 * Update value of total object.
 *
 * @param {number} rowIndex
 * @param {number} columnIndex
 * @param {object} valueObject
 * @param {object} totalObject
 */
PivotTable.prototype.updateValueTotal = function (rowIndex, columnIndex, valueObject, totalObject) {
    if (!totalObject[rowIndex]) {
        totalObject[rowIndex] = {};
    }

    if (!totalObject[rowIndex][columnIndex]) {
        totalObject[rowIndex][columnIndex] = {
            value: 0,
            counter: 0,
            empty: 0,
            numerator: 0,
            denominator: 0,
            factor: 0,
            multiplier: 0,
            divisor: 0,
            totalAggregationType: '',
            valueType: null
        };
    }

    if (valueObject.value === null) {
        totalObject[rowIndex][columnIndex].empty++;
    }

    totalObject[rowIndex][columnIndex].counter++;

    (0, _PivotTableUtils.addMergeValueObject)(totalObject[rowIndex][columnIndex], valueObject);
};

PivotTable.prototype.valueLookupInsert = function (valueObject, rowIndex, columnIndex) {
    if (!this.valueLookup[rowIndex]) {
        this.valueLookup[rowIndex] = {};
    }

    this.valueLookup[rowIndex][columnIndex] = valueObject;

    this.valueCounter += 1;
};

PivotTable.prototype.hasDxInColumns = function () {
    return this.colAxis && Array.isArray(this.colAxis.dimensionNames) && this.colAxis.dimensionNames.includes('dx');
};

PivotTable.prototype.hasDxInRows = function () {
    return this.rowAxis && Array.isArray(this.rowAxis.dimensionNames) && this.rowAxis.dimensionNames.includes('dx');
};

PivotTable.prototype.isSubTotalPosition = function (rowIndex, columnIndex) {
    return Boolean(this.rowAxis.isSubTotalPosition(rowIndex) || this.colAxis.isSubTotalPosition(columnIndex));
};

PivotTable.prototype.isTotalPosition = function (rowIndex, columnIndex) {
    return Boolean(this.rowAxis.isTotalPosition(rowIndex) || this.colAxis.isTotalPosition(columnIndex));
};

PivotTable.prototype.isSubTotalOrTotalPosition = function (rowIndex, columnIndex) {
    return this.isSubTotalPosition(rowIndex, columnIndex) || this.isTotalPosition(rowIndex, columnIndex);
};

/**
 * Initializes value and total lookup tables.
 * TODO: ugly function
 */
PivotTable.prototype.initializeLookups = function () {
    var tableRowSize = this.rowAxis.actualSize;
    var tableColumnSize = this.colAxis.actualSize;

    if (this.rowAxis.doTotals && tableColumnSize !== 1) {
        tableRowSize -= 1;
    }

    if (this.colAxis.doTotals && tableRowSize !== 1) {
        tableColumnSize -= 1;
    }

    var totalMap = {};
    for (var rowIndex = 0; rowIndex < tableRowSize; rowIndex += this.rowAxis.isSubTotalPosition(rowIndex + 1) ? 2 : 1) {
        for (var columnIndex = 0; columnIndex < tableColumnSize; columnIndex += this.colAxis.isSubTotalPosition(columnIndex + 1) ? 2 : 1) {

            var valueObject = this.getValueObject(rowIndex, columnIndex);

            var nextRowTotalIndex = this.rowAxis.getNextTotalPosition();
            var nextColumnTotalIndex = this.colAxis.getNextTotalPosition();

            var nextRowSubTotalIndex = this.rowAxis.getNextSubTotalPosition(rowIndex);
            var nextColumnSubTotalIndex = this.colAxis.getNextSubTotalPosition(columnIndex);

            // insert value
            if (valueObject) {
                this.valueLookupInsert(valueObject.empty ? null : valueObject, rowIndex, columnIndex);
            }

            // add to totals if number
            if (valueObject.valueType === _PivotTableConstants.NUMBER_VALUE_TYPE || !valueObject.valueType && (0, _isNumeric2.default)(valueObject.value)) {

                // used to calculate percentages and check for empties
                if (!valueObject.empty) {

                    if (!this.rowAxis.totalLookup[rowIndex]) this.rowAxis.totalLookup[rowIndex] = 0;
                    if (!this.colAxis.totalLookup[columnIndex]) this.colAxis.totalLookup[columnIndex] = 0;
                    if (!this.rowAxis.totalLookup[nextRowTotalIndex]) this.rowAxis.totalLookup[nextRowTotalIndex] = 0;
                    if (!this.colAxis.totalLookup[nextColumnTotalIndex]) this.colAxis.totalLookup[nextColumnTotalIndex] = 0;

                    this.rowAxis.totalLookup[rowIndex] += valueObject.value;
                    this.colAxis.totalLookup[columnIndex] += valueObject.value;

                    this.rowAxis.totalLookup[nextRowTotalIndex] += valueObject.value;
                    this.colAxis.totalLookup[nextColumnTotalIndex] += valueObject.value;

                    if (this.rowAxis.doSubTotals) {
                        if (!this.rowAxis.totalLookup[nextRowSubTotalIndex]) this.rowAxis.totalLookup[nextRowSubTotalIndex] = 0;
                        this.rowAxis.totalLookup[nextRowSubTotalIndex] += valueObject.value;
                    }

                    if (this.colAxis.doSubTotals) {
                        if (!this.colAxis.totalLookup[nextColumnSubTotalIndex]) this.colAxis.totalLookup[nextColumnSubTotalIndex] = 0;
                        this.colAxis.totalLookup[nextColumnSubTotalIndex] += valueObject.value;
                    }
                }

                // calculate sub totals
                if (this.rowAxis.doSubTotals) {
                    this.updateValueTotal(nextRowSubTotalIndex, columnIndex, valueObject, totalMap);
                }

                if (this.colAxis.doSubTotals) {
                    this.updateValueTotal(rowIndex, nextColumnSubTotalIndex, valueObject, totalMap);
                }

                // calculate totals
                if (this.rowAxis.doTotals) {
                    this.updateValueTotal(nextRowTotalIndex, columnIndex, valueObject, totalMap);
                }

                if (this.colAxis.doTotals) {
                    this.updateValueTotal(rowIndex, nextColumnTotalIndex, valueObject, totalMap);
                }

                // calculate intersection totals
                if (this.rowAxis.doTotals && this.colAxis.doTotals) {
                    this.updateValueTotal(nextRowTotalIndex, nextColumnTotalIndex, valueObject, totalMap);
                }

                if (this.colAxis.doSubTotals && this.rowAxis.doSubTotals) {
                    this.updateValueTotal(nextRowSubTotalIndex, nextColumnSubTotalIndex, valueObject, totalMap);
                }

                if (this.rowAxis.doTotals && this.colAxis.doSubTotals) {
                    this.updateValueTotal(nextRowTotalIndex, nextColumnSubTotalIndex, valueObject, totalMap);
                }

                if (this.colAxis.doTotals && this.rowAxis.doSubTotals) {
                    this.updateValueTotal(nextRowSubTotalIndex, nextColumnTotalIndex, valueObject, totalMap);
                }
            }
        }
    }

    var rowTotalIndices = Object.keys(totalMap);

    for (var i = 0; i < rowTotalIndices.length; i++) {

        var columnTotalIndicies = Object.keys(totalMap[rowTotalIndices[i]]);

        for (var j = 0; j < columnTotalIndicies.length; j++) {

            var _rowIndex4 = parseInt(rowTotalIndices[i]);
            var _columnIndex3 = parseInt(columnTotalIndicies[j]);

            if (!this.valueLookup[_rowIndex4]) {
                this.valueLookup[_rowIndex4] = {};
            }

            if (totalMap[_rowIndex4][_columnIndex3].counter !== totalMap[_rowIndex4][_columnIndex3].empty) {
                var _totalMap$_rowIndex4$ = totalMap[_rowIndex4][_columnIndex3],
                    value = _totalMap$_rowIndex4$.value,
                    numerator = _totalMap$_rowIndex4$.numerator,
                    denominator = _totalMap$_rowIndex4$.denominator,
                    factor = _totalMap$_rowIndex4$.factor,
                    multiplier = _totalMap$_rowIndex4$.multiplier,
                    divisor = _totalMap$_rowIndex4$.divisor,
                    counter = _totalMap$_rowIndex4$.counter,
                    totalAggregationType = _totalMap$_rowIndex4$.totalAggregationType,
                    valueType = _totalMap$_rowIndex4$.valueType;


                var total = null;

                if (this.isSubTotalOrTotalPosition(_rowIndex4, _columnIndex3)) {
                    if ((0, _isNumeric2.default)(value)) {
                        switch (totalAggregationType) {
                            case _PivotTableConstants2.AVERAGE_AGGREGATION_TOTAL:
                                {
                                    total = this.getAverageTotal(numerator || value, denominator, multiplier, divisor);
                                    break;
                                }
                            default:
                                total = value;
                        }
                    }
                }

                if (this.doSortableColumnHeaders()) {
                    var totalIdComb = new _ResponseRowIdCombination.ResponseRowIdCombination(this.refs, [_PivotTableConstants.TOTAL_SORT, this.rowAxis.ids[i]]);
                    this.idValueMap[totalIdComb.get()] = total;
                }

                this.valueLookupInsert({ value: total, valueType: _PivotTableConstants.NUMBER_VALUE_TYPE }, _rowIndex4, _columnIndex3);
            }
        }
    }

    this.totalMap = totalMap;
};

/**
 * Returns row index constrained against the height of the table.
 *
 * @param {number} rowIndex
 * @returns {number}
 */
PivotTable.prototype.constrainHeight = function (rowIndex) {
    return Math.min(this.getRowAxisSize() - this.rowAxis.numberOfEmpty, rowIndex);
};
/**
 * Returns column index constrained against the width of the table.
 *
 * @param {number} columnIndex
 * @returns {number}
 */
PivotTable.prototype.constrainWidth = function (columnIndex) {
    return Math.min(this.getColumnAxisSize() - this.colAxis.numberOfEmpty, columnIndex);
};

/**
 * Builds table filter.
 *
 * @param {number} span
 * @returns {array}
 */
PivotTable.prototype.buildTableFilter = function (span) {

    if (!this.filters) {
        return [];
    }

    var recordNames = this.filters.getRecordNames(false, this.response, true);

    return [[this.buildHtmlCell(new _PivotTableCells.FilterCell(recordNames, this.getTopBarSpan(span)))]];
};

/**
 * Builds table title.
 *
 * @param {number} span
 * @returns
 */
PivotTable.prototype.buildTableTitle = function (span) {

    if (!this.title) {
        return [];
    }

    return [[this.buildHtmlCell(new _PivotTableCells.FilterCell(this.title, this.getTopBarSpan(span)))]];
};

/**
 * Gets legends.
 *
 * @param {string} dxId
 * @returns
 */
PivotTable.prototype.getLegends = function (dxId) {
    return this.appManager.getLegendSetById(this.getLegendSetId(dxId)).legends;
};

/**
 * Gets legend set.
 *
 * @param {string} dxId
 * @returns
 */
PivotTable.prototype.getLegendSet = function (dxId) {

    if (!dxId) {
        return null;
    }

    return this.response.metaData.items[dxId].legendSet;
};

/**
 * Converts table cell to html string
 *
 * @param {object} config
 * @returns {string}
 */
PivotTable.prototype.buildHtmlCell = function (cell) {

    // validation
    if (!(0, _isObject2.default)(cell) || cell.hidden || cell.collapsed) {
        return '';
    }

    var style = '';

    if ((0, _isString2.default)(cell.sort)) {
        this.sortableIdObjects.push({
            id: cell.sort,
            uuid: cell.uuid
        });
    }

    // TOOD: change to use instanceof
    if (cell.type === _PivotTableConstants.VALUE_CELL) {
        this.valueUuids.push(cell.uuid);
    }

    if (cell.type === _PivotTableConstants.DIMENSION_CELL) {
        this.dimensionUuids.push(cell.uuid);
    }

    if (this.doClipping()) {
        style += '\n            min-width:' + cell.width + 'px!important;\n            min-height:' + cell.height + 'px!important;\n            max-width:' + cell.width + 'px!important;\n            max-height:' + cell.height + 'px!important;\n            width:' + cell.width + 'px!important;\n            height:' + cell.height + 'px!important;\n            white-space: nowrap!important;\n            overflow: hidden!important;\n            text-overflow: ellipsis!important;\n        ';
    }

    style += cell.style ? cell.style : '';

    if (cell.isValue) {

        var legends = void 0;
        var cellBackgroundColor = void 0;

        if (!this.options.unclickable) {
            cell.cls += ' pointer';
        }

        if (this.doLegendDisplayByDataItem() && this.getLegendSet(cell.dxId)) {
            legends = this.getLegends(cell.dxId);
        } else {
            legends = this.legendSet ? this.legendSet.legends : [];
        }

        for (var i = 0; i < legends.length; i++) {
            if ((0, _numberConstrain2.default)(cell.value, legends[i].startValue, legends[i].endValue) === cell.value) {
                cellBackgroundColor = legends[i].color;
            }
        }

        if (this.doLegendDisplayStyleFill() && cellBackgroundColor) {
            style += '\n                background-color:' + cellBackgroundColor + ';\n                color:' + ((0, _colorUtils.isColorBright)(cellBackgroundColor) ? _PivotTableConstants.BLACK_COLOR : _PivotTableConstants.WHITE_COLOR) + ';\n            ';
        }

        if (this.doLegendDisplayStyleText() && cellBackgroundColor) {
            style += 'color:' + cellBackgroundColor + ';';
        }
    }

    return '\n        <td data-ou-id="' + (cell.ouId || '') + '"\n            data-period-id="' + (cell.peId || '') + '"\n            class="' + cell.cls + '"\n            id="' + (cell.uuid || '') + '"\n            title="' + (cell.title || '') + '"\n            style="' + style + '"\n            colspan="' + (cell.colSpan || '') + '"\n            rowSpan="' + (cell.rowSpan || '') + '"\n        >\n            ' + cell.displayValue + '\n        </td>\n    ';
};

/**
 * Builds top padding row
 *
 * @returns {string}
 */
PivotTable.prototype.buildTopPaddingHtmlRow = function () {
    var cell = this.buildTopPaddingHtmlCell();

    if (cell) {
        return '<tr>' + cell + '</tr>';
    }

    return '';
};

/**
 * Builds bottom padding row
 *
 * @returns {string}
 */
PivotTable.prototype.buildBottomPaddingHtmlRow = function () {
    var cell = this.buildBottomPaddingHtmlCell();

    if (cell) {
        return '<tr>' + cell + '</tr>';
    }

    return '';
};

/**
 * Builds top padding cell
 *
 * @returns {string}
 */
PivotTable.prototype.buildTopPaddingHtmlCell = function () {

    var padding = this.rowStart * this.options.cellHeight;

    if (!this.options.skipTitle && this.rowStart > 0) {
        padding += this.options.cellHeight;
    }

    var cell = new _PivotTableCells.TopPaddingCell(padding);

    return this.buildHtmlCell(cell);
};

/**
 * Builds bottom padding cell
 *
 * @returns {string}
 */
PivotTable.prototype.buildBottomPaddingHtmlCell = function () {
    var padding = (this.rowAxis.actualSize - this.rowEnd - this.rowAxis.numberOfEmpty) * this.options.cellHeight;
    var cell = new _PivotTableCells.BottomPaddingCell(padding);

    return this.buildHtmlCell(cell);
};

/**
 * Builds left padding cell
 *
 * @returns {string}
 */
PivotTable.prototype.buildLeftPaddingHtmlCell = function () {
    var padding = this.columnStart * this.options.cellWidth;
    var cell = new _PivotTableCells.LeftPaddingCell(padding);

    return this.buildHtmlCell(cell);
};

/**
 * Builds right padding cell
 *
 * @returns {string}
 */
PivotTable.prototype.buildRightPaddingHtmlCell = function () {
    var padding = (this.colAxis.actualSize - this.columnEnd - this.colAxis.numberOfEmpty) * this.options.cellWidth;
    var cell = new _PivotTableCells.RightPaddingCell(padding);

    return this.buildHtmlCell(cell);
};

/**
 * Build html table
 *
 * @returns {string}
 */
PivotTable.prototype.buildHtmlTable = function () {
    var _this = this;

    var cls = 'pivot user-select';
    var style = '\n        display:flex!important;\n        overflow:visible;\n    ';

    cls += this.displayDensity ? ' displaydensity-' + this.displayDensity : '';
    cls += this.fontSize ? ' fontsize-' + this.fontSize : '';

    var htmlRowArray = (0, _arrayClean2.default)([].concat(this.options.skipTitle || this.rowStart > 0 ? [] : this.buildTableTitle(this.table[0].length), this.buildTableFilter(this.table[0].length), this.table.map(function (row) {
        return row.map(function (cell) {
            return _this.buildHtmlCell(cell);
        });
    })));

    var htmlRowString = htmlRowArray.reduce(function (rows, row) {
        return rows += row ? '\n            <tr>\n                ' + (_this.doClipping() ? _this.buildLeftPaddingHtmlCell() : '') + '\n                ' + row.join('') + '\n                ' + (_this.doClipping() ? _this.buildRightPaddingHtmlCell() : '') + '\n            </tr>\n        ' : '';
    }, '');

    return '\n        <div style="' + style + '">\n            <table class="' + cls + '">\n                <tbody>\n                    ' + (this.doClipping() ? this.buildTopPaddingHtmlRow() : '') + '\n                    ' + htmlRowString + '\n                    ' + (this.doClipping() ? this.buildBottomPaddingHtmlRow() : '') + '\n                </tbody>\n            </table>\n        </div>\n    ';
};

// table actions
/**
 * Builds the initial table.
 *
 * @param {number} [columnStart=0]
 * @param {number} [rowStart=0]
 */
PivotTable.prototype.build = function () {
    var columnStart = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    var rowStart = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;


    this.setColumnStart(columnStart);
    this.setRowStart(rowStart);

    var columnEnd = this.getColumnEnd(columnStart);
    var rowEnd = this.getRowEnd(rowStart);

    this.setColumnEnd(columnEnd);
    this.setRowEnd(rowEnd);

    var rowDimension = this.buildRowDimension();
    var columnDimension = this.buildColumnDimension();

    var values = this.buildValueTable();

    for (var rowIndex = 0; rowIndex < rowDimension.length; rowIndex++) {
        var _rowDimension$rowInde;

        (_rowDimension$rowInde = rowDimension[rowIndex]).push.apply(_rowDimension$rowInde, _toConsumableArray(values[rowIndex]));
    }

    if (!(0, _isArray2.default)(columnDimension[0])) {
        columnDimension = [columnDimension];
    }

    // transpose columnDimension
    if (this.colAxis.type) {
        columnDimension = columnDimension[0].map(function (column, index) {
            return columnDimension.map(function (row) {
                return row[index];
            });
        });
    }

    this.table = columnDimension.concat(rowDimension);
};

/**
 * Renders table in form of html string.
 *
 * @returns {string}
 */
PivotTable.prototype.render = function () {

    // used to bind mouse events
    this.valueUuids = [];
    this.dimensionUuids = [];
    this.sortableIdObjects = [];

    if (this.doHideEmptyRows() && (0, _isEmpty2.default)(this.rowAxis.renderLookup) && this.doHideEmptyColumns() && (0, _isEmpty2.default)(this.colAxis.renderLookup)) {
        return 'There is no data to display, and empty rows and columns are hidden.';
    }

    if (this.doHideEmptyRows() && (0, _isEmpty2.default)(this.rowAxis.renderLookup)) {
        return 'There is no data to display, and empty rows are hidden.';
    }

    if (this.doHideEmptyColumns() && (0, _isEmpty2.default)(this.colAxis.renderLookup)) {
        return 'There is no data to display, and empty columns are hidden.';
    }

    // update the span of the corner dimension
    this.updateCornerDimensionSpan();

    // update the span of the column dimension
    this.updateColumnDimensionSpan();

    // update the span of the row dimension
    this.updateRowDimensionSpan();

    // return table converted to HTML
    return this.buildHtmlTable();
};

/**
 * Updates the table given a new column start and row start.
 *
 * @param {number} columnStart
 * @param {number} rowStart
 * @returns {string}
 */
PivotTable.prototype.update = function (columnStart, rowStart) {

    // decrease rowStart by one if top title is present
    if (rowStart > 0 && !this.options.skipTitle) {
        rowStart -= 1;
    }

    var columnEnd = this.getColumnEnd(columnStart);
    var rowEnd = this.getRowEnd(rowStart);

    while (columnStart !== this.columnStart || rowStart !== this.rowStart || columnEnd !== this.columnEnd || rowEnd !== this.rowEnd) {
        this.applyChange(columnStart, columnEnd, rowStart, rowEnd);
    }

    // rerender table
    return this.render();
};

/**
 * Applies incremental changes to the table.
 *
 * @param {number} columnStart new start column index
 * @param {number} columnEnd new end column index
 * @param {number} rowStart new start row index
 * @param {number} rowEnd new end row index
 */
PivotTable.prototype.applyChange = function (columnStart, columnEnd, rowStart, rowEnd) {

    if (this.columnStart > columnStart) {
        this.columnStart--;
        this.addLeftColumn(this.columnStart, this.rowStart, this.rowEnd);
    }

    if (this.columnEnd < columnEnd) {
        this.setColumnEnd(this.columnEnd + 1);
        this.addRightColumn(this.columnEnd, this.rowStart, this.rowEnd);
    }

    if (this.rowStart > rowStart) {
        this.rowStart--;
        this.addTopRow(this.rowStart, this.columnStart, this.columnEnd);
    }

    if (this.rowEnd < rowEnd) {
        this.setRowEnd(this.rowEnd + 1);
        this.addBottomRow(this.rowEnd, this.columnStart, this.columnEnd);
    }

    if (this.rowStart < rowStart) {
        this.rowStart++;
        this.deleteTopRow();
    }

    if (this.rowEnd > rowEnd) {
        this.setRowEnd(this.rowEnd - 1);
        this.deleteBottomRow();
    }

    if (this.columnStart < columnStart) {
        this.columnStart++;
        this.deleteLeftColumn();
    }

    if (this.columnEnd > columnEnd) {
        this.setColumnEnd(this.columnEnd - 1);
        this.deleteRightColumn();
    }
};

/**
 * Builds value table based on given parameters
 *
 * @param {number} rowStart
 * @param {number} rowEnd
 * @param {number} columnStart
 * @param {number} columnEnd
 * @returns {array}
 */
PivotTable.prototype.buildValueTable = function () {

    var rowEnd = this.rowEnd - this.getNumberOfVisibleColumnDimensions();
    var columnEnd = this.columnEnd - this.getNumberOfVisibleRowDimensions();

    var rowSize = rowEnd - this.rowStart;
    var columnSize = columnEnd - this.columnStart;

    var table = [];

    if (this.doClipping()) {
        rowSize = Math.min(rowSize, this.rowAxis.actualSize);
        columnSize = Math.min(columnSize, this.colAxis.actualSize);
    }

    for (var i = 0, rowIndex = this.rowStart; i <= rowSize; i++, rowIndex++) {

        table.push([]);

        for (var j = 0, columnIndex = this.columnStart; j <= columnSize; j++, columnIndex++) {
            table[i].push(this.buildValueCell(columnIndex, rowIndex));
        }
    }

    return table;
};

/**
 * Updates row axis dimension spans based on current render area.
 *
 */
PivotTable.prototype.updateRowDimensionSpan = function () {

    if (!this.rowAxis.type) {
        return;
    }

    // the max span allowed for all columns combined
    var rowSpanLimit = this.rowEnd - this.rowStart + 1 - Math.max(0, this.colAxis.dims - this.rowStart);

    // number of visible row dimensions
    var visibleRowDimensions = this.getNumberOfVisibleRowDimensions();

    // number of visible column dimensions
    var visibleColumnDimensions = this.getNumberOfVisibleColumnDimensions();

    // number offset by column axis dimension
    var rowIndexOffset = Math.max(0, this.rowStart - this.colAxis.dims);

    for (var i = 0, columnIndex = this.columnStart; i < visibleRowDimensions; i++, columnIndex++) {

        var lastId = null;
        var rowSpanCounter = 0;

        for (var j = visibleColumnDimensions, rowIndex = rowIndexOffset; j < this.table.length; j++, rowIndex++) {

            var cell = this.table[j][i];

            if (!this.isCellValid(cell) || this.rowAxis.isPositionHidden(rowIndex)) {
                continue;
            }

            if (this.isDimensionTotalCell(cell)) {
                rowSpanCounter += 1;
                cell.hidden = i !== 0;
                continue;
            }

            // hide all but the first cell which share the same id
            if (cell.hidden = lastId === cell.oldestSibling.uuid) {
                lastId = cell.oldestSibling.uuid;
                continue;
            }

            if (this.rowAxis.span[columnIndex] === 1) {
                cell.rowSpan = 1;
            } else {
                cell.rowSpan = this.rowAxis.getDimensionSpan(rowIndex, columnIndex);
            }

            // limit row span to height of table
            if (cell.rowSpan + rowSpanCounter > rowSpanLimit) {
                cell.rowSpan = rowSpanLimit - rowSpanCounter;
            }

            lastId = cell.oldestSibling.uuid;
            rowSpanCounter += cell.rowSpan;
        }
    }
};

/**
 * Updates column axis dimension span based on current render area.
 *
 */
PivotTable.prototype.updateColumnDimensionSpan = function () {

    if (!this.colAxis.type) {
        return;
    }

    // the max span allowed for all columns combined
    var colSpanLimit = this.columnEnd - this.columnStart + 1 - Math.max(0, this.rowAxis.dims - this.columnStart);

    // number of visible column dimensions
    var visibleColumnDimensions = this.getNumberOfVisibleColumnDimensions();

    // number of visible row dimensions
    var visibleRowDimensions = this.getNumberOfVisibleRowDimensions();

    // number offset by row axis dimension
    var columnIndexOffset = Math.max(0, this.columnStart - this.rowAxis.dims);

    for (var i = 0, rowIndex = this.rowStart; i < visibleColumnDimensions; i++, rowIndex++) {

        var lastId = null;
        var colSpanCounter = 0;

        for (var j = visibleRowDimensions, columnIndex = columnIndexOffset; j < this.table[i].length; j++, columnIndex++) {

            var cell = this.table[i][j];

            // check cell validity and if its hidden
            if (!this.isCellValid(cell) || this.colAxis.isPositionHidden(columnIndex)) {
                continue;
            }

            // if a (sub)total cell, top most cell will span entier dimension
            if (this.isDimensionTotalCell(cell)) {
                colSpanCounter += 1;
                cell.hidden = i !== 0;
                continue;
            }

            // hide all but the first cell which share the same id
            if (cell.hidden = lastId === cell.oldestSibling.uuid) {
                lastId = cell.oldestSibling.uuid;
                continue;
            }

            // get colSpan based cell location
            if (this.colAxis.span[rowIndex] === 1) {
                cell.colSpan = 1;
            } else {
                cell.colSpan = this.colAxis.getDimensionSpan(columnIndex, rowIndex);
            }

            // limit col span to width of table
            if (cell.colSpan + colSpanCounter > colSpanLimit) {
                cell.colSpan = colSpanLimit - colSpanCounter;
            }

            lastId = cell.oldestSibling.uuid;
            colSpanCounter += cell.colSpan;
        }
    }
};

/**
 * Updates corner axis dimension span based on current render area.
 *
 */
PivotTable.prototype.updateCornerDimensionSpan = function () {

    // number of visible column dimensions
    var visibleColumnDimensions = this.getNumberOfVisibleColumnDimensions();

    // number of visible row dimensions
    var visibleRowDimensions = this.getNumberOfVisibleRowDimensions();

    for (var rowIndex = 0; rowIndex < visibleRowDimensions; rowIndex++) {
        for (var columnIndex = 0; columnIndex < visibleColumnDimensions; columnIndex++) {

            // get cell from table
            var cell = this.table[columnIndex][rowIndex];

            // check that cell is valid
            if (!this.isCellValid(cell)) {
                continue;
            }

            // labeled cells will always be visible as they have a row/col span of 1
            if (cell.type === _PivotTableConstants.LABELED_CELL) {
                cell.hidden = false;
            }

            // all empty cells will be hidden except for the upper left which spans the entire corner
            if (cell.type === _PivotTableConstants.EMPTY_CELL) {
                cell.hidden = !(rowIndex === 0 && columnIndex === 0);
            }
        }
    }
};

/**
 * Function used to update table on scrolling
 *
 * @param {function} updateFn
 * @param {number} scrollTop
 * @param {number} scrollLeft
 * @param {function} [callback=Function.prototype]
 */
PivotTable.prototype.scrollHandler = function (updateFn, scrollTop, scrollLeft) {
    var callback = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : Function.prototype;


    // calculate number of rows and columns to render
    var rowLength = Math.floor(scrollTop / this.options.cellHeight);
    var columnLength = Math.floor(scrollLeft / this.options.cellWidth);

    var offset = rowLength === 0 ? 0 : 1;

    // only update if row/column has gone off screen
    if (this.rowStart + offset !== rowLength || this.columnStart !== columnLength) {
        updateFn(this.update(columnLength, rowLength));
        callback();
    }
};

/**
 * Function used to update table on window resize
 *
 * @param {function} updateFn
 * @param {number} newWidth
 * @param {number} newHeight
 * @param {function} [callback=Function.prototype]
 */
PivotTable.prototype.resizeHandler = function (updateFn, newWidth, newHeight) {
    var callback = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : Function.prototype;


    // calculate number of rows and columns to render
    var rowLength = Math.floor(newHeight / this.options.cellHeight);
    var columnLength = Math.floor(newWidth / this.options.cellWidth);

    var offset = rowLength === 0 ? 0 : 1;

    // only update if row/column has gone off screen
    if (this.rowEnd - this.rowStart !== rowLength + offset || this.columnEnd - this.columnStart !== columnLength + offset) {
        updateFn(this.update(this.columnStart, this.rowStart));
        this.setViewportSize(newWidth, newHeight);
        callback();
    }
};

PivotTable.prototype.getSingleValue = function () {
    if (this.valueLookup[0] && this.valueLookup[0][0] !== null && typeof this.valueLookup[0][0] !== 'undefined') {
        return this.valueLookup[0][0];
    }
};

PivotTable.prototype.getSubtitle = function () {

    var subtitle = '';

    if (this.rowAxis) {
        for (var i = 0; i < this.rowAxis.objects.all.length; i++) {
            for (var j = 0; j < this.rowAxis.objects.all[i].length; j++) {
                if (subtitle.length !== 0) {
                    subtitle += ', ';
                }
                subtitle += this.rowAxis.objects.all[i][j].displayValue;
            }
        }
    }

    if (this.colAxis) {
        for (var _i = 0; _i < this.colAxis.objects.all.length; _i++) {
            for (var _j = 0; _j < this.colAxis.objects.all[_i].length; _j++) {
                if (subtitle.length !== 0) {
                    subtitle += ', ';
                }
                subtitle += this.colAxis.objects.all[_i][_j].displayValue;
            }
        }
    }

    return subtitle;
};

PivotTable.prototype.renderCard = function () {

    var value = this.getSingleValue();

    var card = '\n        <span style="display:block;font-size:8em;">\n            ' + (value ? value : 'Empty') + '\n        </span>\n    ';

    var title = '\n        <span style="display:block;fontcolor:#555;font-size:18px;">\n            ' + (this.title ? this.title : '') + '\n        </span>\n    ';

    var subtitle = '\n        <span style="display:block;fontcolor:#555;font-size:14px;">\n            ' + this.getSubtitle() + '\n        </span>\n    ';

    return '\n        <div style="width=100%;height:100%;display:flex;flex-direction:column;justify-content:center;align-items:center;">\n            ' + card + ' <br />\n            ' + title + '\n            ' + subtitle + '\n        </div>\n    ';
};
//# sourceMappingURL=PivotTable.js.map