'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.PivotTableAxis = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _spanTypes;

var _isArray = require('d2-utilizr/lib/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _arrayContains = require('d2-utilizr/lib/arrayContains');

var _arrayContains2 = _interopRequireDefault(_arrayContains);

var _arrayPluck = require('d2-utilizr/lib/arrayPluck');

var _arrayPluck2 = _interopRequireDefault(_arrayPluck);

var _arraySort = require('d2-utilizr/lib/arraySort');

var _arraySort2 = _interopRequireDefault(_arraySort);

var _arrayRepeat = require('d2-utilizr/lib/arrayRepeat');

var _arrayRepeat2 = _interopRequireDefault(_arrayRepeat);

var _uuid = require('d2-utilizr/lib/uuid');

var _uuid2 = _interopRequireDefault(_uuid);

var _isNumber = require('d2-utilizr/lib/isNumber');

var _isNumber2 = _interopRequireDefault(_isNumber);

var _PivotTableConstants = require('./PivotTableConstants');

var _PivotTableUtils = require('./PivotTableUtils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var spanTypes = (_spanTypes = {}, _defineProperty(_spanTypes, _PivotTableConstants.COLUMN_AXIS, 'colSpan'), _defineProperty(_spanTypes, _PivotTableConstants.ROW_AXIS, 'rowSpan'), _spanTypes);

var ignoreKeys = ['dy', 'longitude', 'latitude'];

var PivotTableAxis = exports.PivotTableAxis = function PivotTableAxis(refs, layout, response, type) {
    var _this = this;

    var options = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};


    this.options = _extends({
        showTotals: false,
        showSubTotals: false,
        hideEmpty: false
    }, options);

    this.spanType = spanTypes[type];
    this.size = 1;
    this.dims = 1;
    this.numberOfEmpty = 0;

    this.response = response;

    this.totalLookup = {};
    this.uuidObjectMap = {};
    this.items = [];
    this.span = [];
    this.ids = [];
    this.dimensionNames = [];

    if (type === _PivotTableConstants.COLUMN_AXIS) {
        this.items = (layout.columns || []).filter(function (dim) {
            return !(0, _arrayContains2.default)(ignoreKeys, dim.dimension);
        });
        this.dimensionNames = layout.columns ? layout.columns.getDimensionNames() : [];
    } else if (type === _PivotTableConstants.ROW_AXIS) {
        this.items = (layout.rows || []).filter(function (dim) {
            return !(0, _arrayContains2.default)(ignoreKeys, dim.dimension);
        });
        this.dimensionNames = layout.rows ? layout.rows.getDimensionNames() : [];
    }

    if (!((0, _isArray2.default)(this.items) && this.items.length)) {
        return;
    }

    this.type = type;

    var aAccFloorWidth = [];

    var dimensionIdsFilterFn = function dimensionIdsFilterFn(ids) {
        return ids.filter(function (id) {
            return !id.includes('EMPTY_UID');
        });
    };

    var dimensionNameIdsMap = layout.getDimensionNameIdsMap(response, layout.hideNaData ? dimensionIdsFilterFn : null);

    var aaUniqueFloorIds = function () {
        var dims = void 0;

        return _this.items.map(function (dimension) {
            if (dimension.sorted) dims = (0, _arrayPluck2.default)(dimension.items, 'id');else dims = dimensionNameIdsMap[dimension.dimension];

            _this.size *= dims.length;
            aAccFloorWidth.push(_this.size);

            return dims;
        });
    }();

    this.dims = aaUniqueFloorIds.length;

    var aaGuiFloorIds = aaUniqueFloorIds.map(function (ids, dimensionIndex) {
        _this.span.push(_this.size / aAccFloorWidth[dimensionIndex]);
        return (0, _arrayRepeat2.default)(ids, aAccFloorWidth[dimensionIndex - 1]);
    });

    var aaAllFloorIds = aaGuiFloorIds.map(function (id, dimensionIndex) {
        return (0, _arrayRepeat2.default)(id, _this.span[dimensionIndex], true);
    });

    var aaAllFloorObjects = new Array(this.dims);

    var _loop = function _loop(dimensionIndex) {

        var siblingPosition = 0;
        var oldestObj = void 0;

        var ids = aaAllFloorIds[dimensionIndex];

        aaAllFloorObjects[dimensionIndex] = ids.map(function (id, positionIndex) {

            var dimensionObject = {
                id: id,
                uuid: (0, _uuid2.default)(),
                dim: dimensionIndex,
                leaf: dimensionIndex === _this.dims - 1,
                axis: type,
                isOrganisationUnit: response.hasIdByDimensionName(id, 'ou')
            };

            _this.uuidObjectMap[dimensionObject.uuid] = dimensionObject;

            if (dimensionIndex !== 0) {
                dimensionObject.parent = aaAllFloorObjects[dimensionIndex - 1][positionIndex];
            }

            if (positionIndex % _this.span[dimensionIndex] === 0) {
                dimensionObject[_this.spanType] = _this.span[dimensionIndex];
                dimensionObject.children = dimensionObject.leaf ? 0 : null;
                dimensionObject.oldest = true;
                dimensionObject.root = dimensionIndex === 0;
                oldestObj = dimensionObject;
                siblingPosition = 0;
            }

            if (aaUniqueFloorIds.length - 1 > dimensionIndex) {
                dimensionObject.children = aaUniqueFloorIds[dimensionIndex + 1].length;
            }

            dimensionObject.oldestSibling = oldestObj;
            dimensionObject.siblingPosition = siblingPosition++;

            return dimensionObject;
        });
    };

    for (var dimensionIndex = 0; dimensionIndex < this.dims; dimensionIndex++) {
        _loop(dimensionIndex);
    }

    var _loop2 = function _loop2(positionIndex, _ids) {
        _ids = aaAllFloorIds.map(function (id) {
            return id[positionIndex];
        });

        if (_ids.length) {
            _this.ids.push(_ids.join('-'));
        }
        ids = _ids;
    };

    for (var positionIndex = 0, ids; positionIndex < this.size; positionIndex++) {
        _loop2(positionIndex, ids);
    }

    // add uuids array to leaves
    if (aaAllFloorObjects.length) {

        // set span to second lowest span number: if aFloorSpan == [15,3,15,1], set span to 3
        var nSpan = this.dims > 1 ? (0, _arraySort2.default)(this.span.slice())[1] : this.size;

        var aAllFloorObjectsLast = aaAllFloorObjects[aaAllFloorObjects.length - 1];

        for (var positionIndex = 0, leaf, parentUuids, obj, leafUuids = []; positionIndex < aAllFloorObjectsLast.length; positionIndex++) {

            leaf = aAllFloorObjectsLast[positionIndex];
            leafUuids.push(leaf.uuid);
            obj = leaf;
            parentUuids = [];

            // get the uuid of the oldest sibling
            while (obj = obj.parent) {
                parentUuids.push(!obj.root && obj.oldestSibling ? obj.oldestSibling.uuid : obj.uuid);
            }

            // add parent uuids to leaf
            leaf.uuids = parentUuids;

            // add uuid for all leaves
            if (leafUuids.length === nSpan) {
                for (var j = positionIndex - nSpan + 1, _leaf; j <= positionIndex; j++) {
                    var _leaf$uuids;

                    _leaf = aAllFloorObjectsLast[j];
                    (_leaf$uuids = _leaf.uuids).push.apply(_leaf$uuids, _toConsumableArray(leafUuids));
                }

                leafUuids = [];
            }
        }
    }

    this.dims = this.dims ? this.dims : 1;

    this.xItems = {
        unique: aaUniqueFloorIds,
        gui: aaGuiFloorIds,
        all: aaAllFloorIds
    };

    this.objects = {
        all: aaAllFloorObjects
    };

    this.doHideEmpty = this.options.hideEmpty && this.type;

    this.doTotals = this.options.showTotals;
    this.doSubTotals = this.options.showSubTotals && this.type && this.dims > 1;

    this.uniqueFactor = this.getUniqueFactor();
    this.spanMap = this.getSpanMap();
};

PivotTableAxis.prototype.isPositionEmpty = function (positionIndex) {
    return !(0, _isNumber2.default)(this.totalLookup[positionIndex]);
};

PivotTableAxis.prototype.isPositionHidden = function (positionIndex) {
    return this.doHideEmpty && this.isPositionEmpty(this.getPositionIndexOffsetHidden(positionIndex));
};

PivotTableAxis.prototype.isSubTotalPosition = function (positionIndex) {
    return this.doSubTotals && (positionIndex + 1) % (this.uniqueFactor + 1) === 0;
};

PivotTableAxis.prototype.isTotalPosition = function (positionIndex) {
    return this.doTotals && positionIndex === this.actualSize - 1;
};

PivotTableAxis.prototype.getPercentage = function (value, positionIndex) {
    return (0, _PivotTableUtils.getPercentageHtml)(value, this.totalLookup[positionIndex]);
};

PivotTableAxis.prototype.getNextTotalPosition = function () {
    return this.actualSize - 1;
};

PivotTableAxis.prototype.getAxisSpan = function (positionIndex, dimensionIndex) {
    return this.spanMap[dimensionIndex][Math.floor(positionIndex / this.span[dimensionIndex])];
};

PivotTableAxis.prototype.initializeRenderLookup = function () {
    this.renderLookup = this.createRenderMap();
};

PivotTableAxis.prototype.getNextSubTotalPosition = function (positionIndex) {

    positionIndex = this.getPositionIndexWithoutTotals(positionIndex);

    return positionIndex + Math.floor(positionIndex / this.uniqueFactor) + (this.uniqueFactor - positionIndex % this.uniqueFactor);
};

PivotTableAxis.prototype.getAxisLabel = function (positionIndex) {

    if (this.dims) {
        return this.response.getNameById(this.dimensionNames[positionIndex]);
    }

    return null;
};

PivotTableAxis.prototype.getDimensionSpan = function (positionIndex, dimensionIndex) {
    return this.span[dimensionIndex] - this.getNumberOfHiddenPreceding(positionIndex, dimensionIndex) - this.getNumberOfHiddenProceeding(positionIndex, dimensionIndex) - this.getDistanceFromLastPositionDimensionUnitActual(positionIndex, dimensionIndex);
};

PivotTableAxis.prototype.getNumberOfHiddenPreceding = function (positionIndex, dimensionIndex) {
    return this.getDistanceFromLastPositionDimensionUnitOffset(positionIndex, dimensionIndex) - this.getDistanceFromLastPositionDimensionUnitActual(positionIndex, dimensionIndex);
};

PivotTableAxis.prototype.getDistanceFromLastPositionDimensionUnitOffset = function (positionIndex, dimensionIndex) {
    var actualPositionIndex = this.getPositionIndexOffsetHidden(positionIndex);
    var normalized = this.getPositionIndexWithoutTotals(actualPositionIndex);

    return normalized % this.span[dimensionIndex];
};

PivotTableAxis.prototype.getDistanceFromLastPositionDimensionUnitActual = function (positionIndex, dimensionIndex) {

    var actualPositionIndex = this.getPositionIndexOffsetHidden(positionIndex);
    var normalized = this.getPositionIndexWithoutTotals(actualPositionIndex);

    return this.normalizePositionIndexOffset(positionIndex) - (this.sumAxisSpanUpToIndex(normalized, dimensionIndex) - this.getAxisSpan(normalized, dimensionIndex));
};

PivotTableAxis.prototype.getNumberOfHiddenProceeding = function (positionIndex, dimensionIndex) {
    return this.getDistanceToNextPositionDimensionUnitOffset(positionIndex, dimensionIndex) - this.getDistanceToNextPositionDimensionUnitActual(positionIndex, dimensionIndex);
};

PivotTableAxis.prototype.getDistanceToNextPositionDimensionUnitOffset = function (positionIndex, dimensionIndex) {

    var actualPositionIndex = this.getPositionIndexOffsetHidden(positionIndex);
    var normalized = this.getPositionIndexWithoutTotals(actualPositionIndex);

    return this.span[dimensionIndex] - normalized % this.span[dimensionIndex];
};

PivotTableAxis.prototype.getDistanceToNextPositionDimensionUnitActual = function (positionIndex, dimensionIndex) {

    var actualPositionIndex = this.getPositionIndexOffsetHidden(positionIndex);
    var normalized = this.getPositionIndexWithoutTotals(actualPositionIndex);

    return this.getAxisSpan(normalized, dimensionIndex) - (this.normalizePositionIndexOffset(positionIndex) - (this.sumAxisSpanUpToIndex(normalized, dimensionIndex) - this.getAxisSpan(normalized, dimensionIndex)));
};

PivotTableAxis.prototype.getPositionIndexOffsetHidden = function (positionIndex) {

    if (this.doHideEmpty) {
        return this.renderLookup[positionIndex];
    }

    return positionIndex;
};

PivotTableAxis.prototype.getPositionIndexWithoutTotals = function (positionIndex) {

    if (this.doSubTotals) {
        return Math.max(0, positionIndex - Math.floor(positionIndex / (this.uniqueFactor + 1)));
    }

    return positionIndex;
};

PivotTableAxis.prototype.normalizePositionIndexOffset = function (positionIndex) {

    if (this.doSubTotals) {
        var actualPositionIndex = this.getPositionIndexOffsetHidden(positionIndex);
        return positionIndex - Math.floor(actualPositionIndex / (this.uniqueFactor + 1));
    }

    return positionIndex;
};

PivotTableAxis.prototype.decrementAxisSpan = function (positionIndex, dimensionIndex) {
    this.spanMap[dimensionIndex][Math.floor(positionIndex / this.span[dimensionIndex])] -= 1;
};

PivotTableAxis.prototype.createRenderMap = function () {

    var map = [];

    for (var positionIndex = 0; positionIndex < this.actualSize; positionIndex++) {

        if (!this.isPositionEmpty(positionIndex) || !this.doHideEmpty) {
            map.push(positionIndex);
            continue;
        }

        this.numberOfEmpty += 1;

        for (var dimensionIndex = 0; dimensionIndex < this.span.length - 1; dimensionIndex++) {
            var normalizedPositionIndex = this.getPositionIndexWithoutTotals(positionIndex);
            this.decrementAxisSpan(normalizedPositionIndex, dimensionIndex);
        }
    }

    return map;
};

PivotTableAxis.prototype.sumAxisSpanUpToIndex = function (positionIndex, dimensionIndex) {

    var sum = 0;

    for (var i = 0; i < Math.floor(positionIndex / this.span[dimensionIndex]) + 1; i++) {
        sum += this.spanMap[dimensionIndex][i];
    }

    return sum;
};

PivotTableAxis.prototype.getSpanMap = function () {

    if (!this.span) {
        this.span = [];
    }

    return Array.from(this.span, function (value) {
        return (0, _PivotTableUtils.defaultProxyGenerator)(value);
    });
};

PivotTableAxis.prototype.getActualSize = function () {

    var size = this.size;

    if (!size) {
        return 1;
    }

    if (this.doSubTotals) {
        size += this.size / this.uniqueFactor;
    }

    if (this.doTotals) {
        size += 1;
    }

    return size;
};

PivotTableAxis.prototype.getUniqueFactor = function () {

    if (this.xItems && this.xItems.unique) {
        return this.xItems.unique.length < 2 ? 1 : this.size / this.xItems.unique[0].length;
    }

    return null;
};
//# sourceMappingURL=PivotTableAxis.js.map