'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.assert = exports.defaultProxyGenerator = exports.addMergeValueObject = exports.addMerge = exports.roundAndStrip = exports.roundIf = exports.getPercentageHtml = exports.getRoundedHtmlValue = exports.getNumberOfDecimals = exports.getDefaultNumberDisplayValue = undefined;

var _numberToFixed = require('d2-utilizr/lib/numberToFixed');

var _numberToFixed2 = _interopRequireDefault(_numberToFixed);

var _isNumber = require('d2-utilizr/lib/isNumber');

var _isNumber2 = _interopRequireDefault(_isNumber);

var _isNumeric = require('d2-utilizr/lib/isNumeric');

var _isNumeric2 = _interopRequireDefault(_isNumeric);

var _PivotTableConstants = require('./PivotTableConstants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getDefaultNumberDisplayValue = exports.getDefaultNumberDisplayValue = function getDefaultNumberDisplayValue(value, skipRounding) {
    return typeof value === null || value === undefined ? '' : skipRounding ? getRoundedHtmlValue(value, _PivotTableConstants.SKIP_ROUNDING_NUMBER_DECIMALS) : getRoundedHtmlValue(value, value < 1 && value > -1 ? _PivotTableConstants.SMALL_NUMBER_DECIMALS : _PivotTableConstants.DEFAULT_NUMBER_DECIMALS);
};

/** @description returns the number of decumal of given float
 *  @param   {number} number
 *  @returns {number}
 *  @deprecated should switch to use function located in d2-utilizr
 */
var getNumberOfDecimals = exports.getNumberOfDecimals = function getNumberOfDecimals(number) {
    var str = new String(number);
    return str.indexOf('.') > -1 ? str.length - str.indexOf('.') - 1 : 0;
};

/** @description returns the rounded value of the given float.
 *  @param   {number} value
 *  @param   {number} [dec=NUMBER_OF_DECIMALS]
 *  @returns {number}
 */
var getRoundedHtmlValue = exports.getRoundedHtmlValue = function getRoundedHtmlValue(value) {
    var dec = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _PivotTableConstants.DEFAULT_NUMBER_DECIMALS;

    return parseFloat(roundAndStrip(value, dec)).toString();
};

/** @description get percentage representation of value.
 *  @param   {number} value
 *  @param   {number} total
 *  @returns {string}
 */
var getPercentageHtml = exports.getPercentageHtml = function getPercentageHtml(value, total) {
    return getRoundedHtmlValue(value / total * 100) + '%';
};

/** @description round number if needed.
 *  @param   {number} number
 *  @param   {number} precision
 *  @returns {number}
 */
var roundIf = exports.roundIf = function roundIf(number, precision) {
    number = parseFloat(number);
    precision = parseFloat(precision);

    if ((0, _isNumber2.default)(number) && (0, _isNumber2.default)(precision)) {
        var numberOfDecimals = getNumberOfDecimals(number);
        return numberOfDecimals > precision ? (0, _numberToFixed2.default)(number, precision) : number;
    }

    return number;
};

/** @description round number if needed.
 *  @param   {number} number
 *  @param   {number} precision
 *  @returns {number}
 */
var roundAndStrip = exports.roundAndStrip = function roundAndStrip(number, precision) {
    var n = parseFloat(number);

    if (!((0, _isNumeric2.default)(number) && (0, _isNumber2.default)(n))) {
        return number;
    }

    if (!(0, _isNumeric2.default)(precision)) {
        return n;
    }

    return n.toFixed(precision);
};

var addMerge = exports.addMerge = function addMerge(a, b) {
    Object.keys(a).forEach(function (key) {
        if (b[key]) {
            a[key] += b[key];
        }
    });
};

var addMergeValueObject = exports.addMergeValueObject = function addMergeValueObject(a, b) {
    Object.keys(a).forEach(function (key) {
        if (Boolean(b[key])) {
            if (typeof a[key] === 'number') {
                a[key] += b[key];
                return;
            } else if (key === 'totalAggregationType') {
                a[key] = !a[key] ? b[key] : a[key] !== b[key] ? _PivotTableConstants.NA_AGGREGATION_TOTAL : a[key];
            }
        }
    });
};

var defaultProxyGenerator = exports.defaultProxyGenerator = function defaultProxyGenerator(defaultReturnValue) {
    return new Proxy({}, { get: function get(target, name) {
            return name in target ? target[name] : defaultReturnValue;
        } });
};

var assert = exports.assert = function assert(condition, message) {
    if (!condition) {
        message = message || "Assertion failed";
        if (typeof Error !== "undefined") {
            throw new Error(message);
        }
        throw message; // Fallback
    }
};
//# sourceMappingURL=PivotTableUtils.js.map