'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.InterpretationWindow = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _SharingWindow = require('./SharingWindow');

var _MentionToolbar = require('./MentionToolbar.js');

var _permissions = require('../util/permissions');

var _dom = require('../util/dom');

var InterpretationWindow = exports.InterpretationWindow = undefined;

exports.InterpretationWindow = InterpretationWindow = function InterpretationWindow(c, sharing, interpretation, success, options) {
    var _ref = options || {},
        _ref$renderText = _ref.renderText,
        renderText = _ref$renderText === undefined ? true : _ref$renderText,
        _ref$renderSharing = _ref.renderSharing,
        renderSharing = _ref$renderSharing === undefined ? true : _ref$renderSharing;

    var appManager = c.appManager,
        uiManager = c.uiManager,
        instanceManager = c.instanceManager,
        i18n = c.i18nManager.get(),
        apiPath = appManager.getApiPath(),
        apiResource = instanceManager.apiResource,
        apiEndpoint = instanceManager.apiEndpoint;

    var textArea = Ext.create('Ext.form.field.TextArea', {
        cls: 'ns-textarea',
        height: 130,
        width: 407,
        fieldStyle: 'padding-left:3px; padding-top:3px',
        emptyText: i18n.write_your_interpretation + '...',
        enableKeyEvents: true,
        value: interpretation ? interpretation.text : undefined,
        mentionToolbar: (0, _MentionToolbar.MentionToolbar)(c),
        listeners: {
            keyup: function keyup(f, e) {
                shareButton.xable();
                this.mentionToolbar.displayMentionSuggestion(f, e);
            },
            keydown: _dom.onMarkdownEditorKeyDown,
            destroy: function destroy(f, e) {
                this.mentionToolbar.destroy();
            }
        }
    });

    var getSharingWithModel = function getSharingWithModel(sharing) {
        return _extends({}, sharing, {
            object: _extends({}, sharing.object, { modelName: "interpretation", name: "" })
        });
    };

    var sharingCmp = renderSharing && sharing ? new _SharingWindow.SharingWindow(c, getSharingWithModel(sharing), true) : null;

    var sharingCt = Ext.create('Ext.container.Container', {
        style: 'padding-top:10px; padding-bottom:25px',
        items: sharingCmp ? sharingCmp.items : []
    });

    var method = interpretation ? 'PUT' : 'POST';

    var interpretationSuccess = function interpretationSuccess(text) {
        if (interpretation) {
            interpretation.text = text;
        }
        if (success) {
            success();
        } else {
            instanceManager.getById(null, function (layout, isFavorite) {
                instanceManager.getReport(layout, isFavorite, false, false, function () {
                    uiManager.unmask();
                });
            });
        }
    };

    var updateSharing = function updateSharing(response, text) {
        var interpretationId = interpretation ? interpretation.id : (response.getResponseHeader('location') || '').split('/').pop(),
            sharingId = sharing.object.id,
            sharingBody = sharingCmp.getBody();

        Ext.Ajax.request({
            url: encodeURI(apiPath + '/sharing?type=interpretation&id=' + interpretationId),
            method: method,
            headers: {
                'Content-Type': 'application/json'
            },
            params: Ext.encode(sharingBody),
            success: function success() {
                // Reload favorite to update interpretation sharings in the global object
                instanceManager.getById(null, function (layout, isFavorite) {
                    instanceManager.getReport(layout, isFavorite, false, false, function () {
                        uiManager.unmask();
                    });
                });
            },
            callback: function callback() {
                if (text) {
                    interpretationSuccess(text);
                }
            }
        });
    };

    var shareButton = Ext.create('Ext.button.Button', {
        text: interpretation ? i18n.update : i18n.share,
        disabled: !interpretation,
        xable: function xable() {
            this.setDisabled(!textArea.getValue());
        },
        handler: function handler() {
            if (sharingCmp) {
                var showSharingError = function showSharingError() {
                    var errorMessage = i18n.validation_error_interpretation_sharing;
                    uiManager.alert(errorMessage);
                };
                var favorite = instanceManager.getStateFavorite();
                var newSharing = sharingCmp.getBody().object;

                if (!(0, _permissions.validateSharing)("interpretation", favorite, newSharing, showSharingError)) {
                    return;
                }
            }

            var text = textArea.getValue();
            var interpretationPath = interpretation ? '/interpretations/' + interpretation.id : '/interpretations/' + apiResource + '/' + instanceManager.getStateFavoriteId();

            if (renderText) {
                Ext.Ajax.request({
                    url: encodeURI(apiPath + interpretationPath),
                    method: method,
                    params: text,
                    headers: { 'Content-Type': 'text/html' },
                    success: function success(response) {
                        sharing ? updateSharing(response, text) : interpretationSuccess(text);
                        textArea.reset();
                        window.destroy();
                    }
                });
            } else if (renderSharing && sharing) {
                updateSharing();
                window.destroy();
            }
        }
    });

    var window = Ext.create('Ext.window.Window', {
        title: renderText ? i18n.write_interpretation : i18n.sharing_settings,
        layout: 'fit',
        bodyStyle: 'padding:4px; background-color:#fff',
        resizable: false,
        destroyOnBlur: true,
        modal: true,
        items: [renderText && textArea, renderSharing && sharingCt].filter(function (item) {
            return item;
        }),
        bbar: {
            cls: 'ns-toolbar-bbar',
            defaults: {
                height: 24
            },
            items: ['->', shareButton]
        },
        listeners: {
            show: function show(w) {
                uiManager.setAnchorPosition(w, 'favoriteButton', { y: 130 });

                uiManager.enableRightClick();

                if (!w.hasDestroyOnBlurHandler) {
                    uiManager.addDestroyOnBlurHandler(w);
                }
            },
            hide: function hide() {
                uiManager.disableRightClick();
            }
        }
    });
    uiManager.reg(window, 'interpretationWindow');

    return window;
};
//# sourceMappingURL=InterpretationWindow.js.map